#include "HX711.h"
#include <SoftwareSerial.h> 
#include <Servo.h>
#include "pitches_h.h"
#include "themes_h.h"

#define RX_PIN 2
#define TX_PIN 3
#define LOADCELL_DOUT_PIN 4
#define LOADCELL_SCK_PIN 5
#define buttonBlue 6
#define buttonBlack 7
#define buttonRed 8
#define servoMotor 9
#define pump 10
#define buzzer 11
#define waterSensor A0

HX711 scale;
Servo servo;
SoftwareSerial bt(2, 3); // RX | TX 

int buttonBlueState = 0;
int buttonBlackState = 0;
int buttonRedState = 0;

int mode = 0;
int flag = 0;
int val = 0;

void setup() {
  Serial.begin(9600);
  
  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);

  bt.begin(9600); 
  Serial.println("Ready to connect. Default password is 1234"); 

  pinMode(buttonBlue, INPUT);
  pinMode(buttonRed, INPUT);
  pinMode(buttonBlack, INPUT);

  pinMode(pump, OUTPUT);
  servo.attach(servoMotor);
}

void pocSong() { 
  
  for (int thisNote = 0; thisNote < (sizeof(Pirates_note)/sizeof(int)); thisNote++) {

    int noteDuration = 1000 / Pirates_duration[thisNote];
    tone(buzzer, Pirates_note[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.05;
    delay(pauseBetweenNotes);
    noTone(buzzer);
  }
}

void despSong() {
  
  for (int thisNote = 0; thisNote < (sizeof(desp_note)/sizeof(int)); thisNote++) {

    int noteDuration = 1000 / desp_duration[thisNote];
    tone(buzzer, desp_note[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.50;
    delay(pauseBetweenNotes);
    noTone(buzzer);
  }
}

void marioSong() {
  
  for (int thisNote = 0; thisNote < (sizeof(MarioUW_note)/sizeof(int)); thisNote++) {

    int noteDuration = 1000 / MarioUW_duration[thisNote];
    tone(buzzer, MarioUW_note[thisNote], noteDuration);
    int pauseBetweenNotes = noteDuration * 1.80;
    delay(pauseBetweenNotes);
    noTone(buzzer);
  }
}

void startMotor(){
  servo.write(180);
}

void stopMotor() {
  servo.write(90);
}

void startPump() {
  digitalWrite(pump, HIGH);
}

void stopPump() {
  digitalWrite(pump, LOW);
}

void weightSensorFunc() {

    int weight = readWeightSensor();
      
    if(weight < 10) {
      startMotor();
    }
    
    if(weight > 80) {
      stopMotor();
    }
    
}

int copyWeight = -10;
int restartWeight = 0;
int motorOn = 0;

void weightSensorFunc2() {

    int weight = readWeightSensor();

    if(restartWeight == 0) {
      
      if(weight < 10) {
        startMotor();
        motorOn = 1;
      }
    
      if(weight > 80) {
        stopMotor();
        motorOn = 0;
      }

      if(motorOn && isStable(weight, copyWeight)) {
        restartWeight = 1;
        stopMotor();
      }

      copyWeight = weight;
    }
      
    
    
}

float readWeightSensor() {
  
  if (scale.is_ready()) {
    
    long reading = scale.read() * 266;
    float grams = (float)reading / 100000.0;
    grams = grams - 169.80;
    
    Serial.print("Grams: ");
    Serial.println(grams);
    
    return grams;
  }
}


void waterSensorFunc() {

  int level = readWaterSensor();

  if(level < 20) {
    startPump();
  }
  if(level > 450) {
    stopPump();
  }

}

int copyLevel = -10;
int pumpOn = 0;
int restartLevel = 0;

void waterSensorFunc2() {

  int level = readWaterSensor();

  if(restartLevel == 0) {
    
    if(level < 20) {
      startPump();
      pumpOn = 1;
    }
    if(level > 450) {
      stopPump();
      pumpOn = 0;
    }

    if(pumpOn && isStable(level, copyLevel)) {
      restartLevel = 1;
      stopPump();
    }
      
    copyLevel = level;
  }
}


int readWaterSensor() {
  
  delay(10);
  val = analogRead(A0);
  
  Serial.print("Level: ");
  Serial.println(val);
  
  return val;
}

void calibrateWeightSensor() {
  
  if (scale.is_ready()) {
    
    long reading = scale.read() * 266;
    float grams = (float)reading / 100000.0;
    grams = grams - 169.80;
    
    Serial.print("Grams: ");
    Serial.println(grams);
  }
}

void calibrateWaterSensor() {
  
  val = analogRead(A0);
  
  Serial.print("Level: ");
  Serial.println(val);
}

int isStable(int current, int previous) {
  if(current < previous)
    return 1;
    
  return 0;
}

void loop() {

  Serial.print("MODE: ");
  Serial.println(mode);

  calibrateWeightSensor();
  calibrateWaterSensor();
  
  buttonBlackState = digitalRead(buttonBlack);

  if(buttonBlackState) {
    mode = (mode + 1) % 3;

    if(mode == 0) {
      pocSong();
    }

    if(mode == 1) {
      despSong();
    }

    if(mode == 2) {
      marioSong();
    }

    // se asigura oprirea sistemului la orice schimbare de mod
    flag = 0;
    stopPump();
    stopMotor();
    
  }
     
  if(mode == 0) {

    buttonBlueState = digitalRead(buttonBlue);
    buttonRedState = digitalRead(buttonRed);
    
    if(buttonRedState) {
      startMotor();
    } else {
      stopMotor();
    }

    if(buttonBlueState) {
      startPump();
    } else {
      stopPump();
    }
  }

  if(mode == 1) {

    if (bt.available()) {
      flag = bt.read();
    }
      
    if (flag == 49) { // 1
      startMotor();
    } 
    
    if (flag == 50) { // 2
      stopMotor();
    }

    if (flag == 51) { // 3
      startPump();
    } 
    
    if (flag == 52) { // 4
      stopPump();
    }
  }

  if(mode == 2) {
    weightSensorFunc();
    waterSensorFunc();
  }
  
}
